package java.awt;

import sun.awt.AWTAccessor;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import sun.awt.HeadlessToolkit;
import sun.awt.SunToolkit;
import sun.awt.AppContext;
import java.security.AccessController;
import java.security.AccessControlContext;
import java.awt.event.ActionListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.peer.TrayIconPeer;

public class TrayIcon
{
    private Image image;
    private String tooltip;
    private PopupMenu popup;
    private boolean autosize;
    private int id;
    private String actionCommand;
    private transient TrayIconPeer peer;
    transient MouseListener mouseListener;
    transient MouseMotionListener mouseMotionListener;
    transient ActionListener actionListener;
    private final AccessControlContext acc;
    
    final AccessControlContext getAccessControlContext() {
        if (this.acc == null) {
            throw new SecurityException("TrayIcon is missing AccessControlContext");
        }
        return this.acc;
    }
    
    private TrayIcon() throws UnsupportedOperationException, HeadlessException, SecurityException {
        this.acc = AccessController.getContext();
        SystemTray.checkSystemTrayAllowed();
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        if (!SystemTray.isSupported()) {
            throw new UnsupportedOperationException();
        }
        SunToolkit.insertTargetMapping(this, AppContext.getAppContext());
        return;
    }
    
    public TrayIcon(Image image) {
        this();
        if (image == null) {
            throw new IllegalArgumentException("creating TrayIcon with null Image");
        }
        this.setImage(image);
        return;
    }
    
    public TrayIcon(Image image, String toolTip) {
        this(image);
        this.setToolTip(toolTip);
    }
    
    public TrayIcon(Image image, String s, PopupMenu popupMenu) {
        this(image, s);
        this.setPopupMenu(popupMenu);
    }
    
    public void setImage(Image image) {
        if (image == null) {
            throw new NullPointerException("setting null Image");
        }
        this.image = image;
        TrayIconPeer peer = this.peer;
        if (peer != null) {
            peer.updateImage();
        }
        return;
    }
    
    public Image getImage() {
        return this.image;
    }
    
    public void setPopupMenu(PopupMenu popup) {
        if (popup == this.popup) {
            return;
        }
        synchronized (TrayIcon.class) {
            if (popup != null) {
                if (popup.isTrayIconPopup) {
                    throw new IllegalArgumentException("the PopupMenu is already set for another TrayIcon");
                }
                popup.isTrayIconPopup = true;
            }
            if (this.popup != null) {
                this.popup.isTrayIconPopup = false;
            }
            this.popup = popup;
        }
        return;
    }
    
    public PopupMenu getPopupMenu() {
        return this.popup;
    }
    
    public void setToolTip(String s) {
        this.tooltip = s;
        TrayIconPeer peer = this.peer;
        if (peer != null) {
            peer.setToolTip(s);
        }
    }
    
    public String getToolTip() {
        return this.tooltip;
    }
    
    public void setImageAutoSize(boolean autosize) {
        this.autosize = autosize;
        TrayIconPeer peer = this.peer;
        if (peer != null) {
            peer.updateImage();
        }
    }
    
    public boolean isImageAutoSize() {
        return this.autosize;
    }
    
    public synchronized void addMouseListener(MouseListener mouseListener) {
        if (mouseListener == null) {
            return;
        }
        this.mouseListener = AWTEventMulticaster.add(this.mouseListener, mouseListener);
        return;
    }
    
    public synchronized void removeMouseListener(MouseListener mouseListener) {
        if (mouseListener == null) {
            return;
        }
        this.mouseListener = AWTEventMulticaster.remove(this.mouseListener, mouseListener);
        return;
    }
    
    public synchronized MouseListener[] getMouseListeners() {
        return (MouseListener[])AWTEventMulticaster.getListeners(this.mouseListener, MouseListener.class);
    }
    
    public synchronized void addMouseMotionListener(MouseMotionListener mouseMotionListener) {
        if (mouseMotionListener == null) {
            return;
        }
        this.mouseMotionListener = AWTEventMulticaster.add(this.mouseMotionListener, mouseMotionListener);
        return;
    }
    
    public synchronized void removeMouseMotionListener(MouseMotionListener mouseMotionListener) {
        if (mouseMotionListener == null) {
            return;
        }
        this.mouseMotionListener = AWTEventMulticaster.remove(this.mouseMotionListener, mouseMotionListener);
        return;
    }
    
    public synchronized MouseMotionListener[] getMouseMotionListeners() {
        return (MouseMotionListener[])AWTEventMulticaster.getListeners(this.mouseMotionListener, MouseMotionListener.class);
    }
    
    public String getActionCommand() {
        return this.actionCommand;
    }
    
    public void setActionCommand(String actionCommand) {
        this.actionCommand = actionCommand;
    }
    
    public synchronized void addActionListener(ActionListener actionListener) {
        if (actionListener == null) {
            return;
        }
        this.actionListener = AWTEventMulticaster.add(this.actionListener, actionListener);
        return;
    }
    
    public synchronized void removeActionListener(ActionListener actionListener) {
        if (actionListener == null) {
            return;
        }
        this.actionListener = AWTEventMulticaster.remove(this.actionListener, actionListener);
        return;
    }
    
    public synchronized ActionListener[] getActionListeners() {
        return (ActionListener[])AWTEventMulticaster.getListeners(this.actionListener, ActionListener.class);
    }
    
    public void displayMessage(String s, String s2, TrayIcon.MessageType messageType) {
        if (s == null && s2 == null) {
            throw new NullPointerException("displaying the message with both caption and text being null");
        }
        TrayIconPeer peer = this.peer;
        if (peer != null) {
            peer.displayMessage(s, s2, messageType.name());
        }
        return;
    }
    
    public Dimension getSize() {
        return SystemTray.getSystemTray().getTrayIconSize();
    }
    
    void addNotify() throws AWTException {
        synchronized (this) {
            if (this.peer == null) {
                Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
                if ((defaultToolkit instanceof SunToolkit)) {
                    this.peer = ((SunToolkit)Toolkit.getDefaultToolkit()).createTrayIcon(this);
                }
                else if ((defaultToolkit instanceof HeadlessToolkit)) {
                    this.peer = ((HeadlessToolkit)Toolkit.getDefaultToolkit()).createTrayIcon(this);
                }
            }
        }
        this.peer.setToolTip(this.tooltip);
    }
    
    void removeNotify() {
        TrayIconPeer peer = null;
        synchronized (this) {
            peer = this.peer;
            this.peer = null;
        }
        if (peer != null) {
            peer.dispose();
        }
    }
    
    void setID(int id) {
        this.id = id;
    }
    
    int getID() {
        return this.id;
    }
    
    void dispatchEvent(AWTEvent currentEventAndMostRecentTime) {
        EventQueue.setCurrentEventAndMostRecentTime(currentEventAndMostRecentTime);
        Toolkit.getDefaultToolkit().notifyAWTEventListeners(currentEventAndMostRecentTime);
        this.processEvent(currentEventAndMostRecentTime);
    }
    
    void processEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof MouseEvent)) {
            switch (awtEvent.getID()) {
                case 500:
                case 501:
                case 502: {
                    this.processMouseEvent((MouseEvent)awtEvent);
                    break;
                }
                case 503: {
                    this.processMouseMotionEvent((MouseEvent)awtEvent);
                    break;
                }
                default: {
                    return;
                }
            }
        }
        else if ((awtEvent instanceof ActionEvent)) {
            this.processActionEvent((ActionEvent)awtEvent);
        }
    }
    
    void processMouseEvent(MouseEvent mouseEvent) {
        MouseListener mouseListener = this.mouseListener;
        if (mouseListener != null) {
            switch (mouseEvent.getID()) {
                case 501: {
                    mouseListener.mousePressed(mouseEvent);
                    break;
                }
                case 502: {
                    mouseListener.mouseReleased(mouseEvent);
                    break;
                }
                case 500: {
                    mouseListener.mouseClicked(mouseEvent);
                    break;
                }
                default: {
                    return;
                }
            }
        }
    }
    
    void processMouseMotionEvent(MouseEvent mouseEvent) {
        MouseMotionListener mouseMotionListener = this.mouseMotionListener;
        if (mouseMotionListener != null && mouseEvent.getID() == 503) {
            mouseMotionListener.mouseMoved(mouseEvent);
        }
    }
    
    void processActionEvent(ActionEvent actionEvent) {
        ActionListener actionListener = this.actionListener;
        if (actionListener != null) {
            actionListener.actionPerformed(actionEvent);
        }
    }
    
    private static native void initIDs();
    
    static {
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        AWTAccessor.setTrayIconAccessor(new AWTAccessor.TrayIconAccessor() {
            @Override
            public void addNotify(TrayIcon trayIcon) throws AWTException {
                trayIcon.addNotify();
            }
            
            @Override
            public void removeNotify(TrayIcon trayIcon) {
                trayIcon.removeNotify();
            }
        });
    }
    
    public enum MessageType
    {
        ERROR, 
        WARNING, 
        INFO, 
        NONE;
    }
}
